/** @invariant size () >= 0 */

public class FixSizeStack
{
    /** @post size () == 0 */

    public FixSizeStack () {
        _storage = new Object [MAX_SIZE];
        _top = 0;
    }

    /**
      * @pre size () < MAX_SIZE - 1
      * @post peek () == object
      * @post size () == $pre (int, size ()) + 1
      * @concurrency sequential
      */

    public void push (Object object) {
        _storage [_top++] = object;
    }

    /**
      * @pre size () > 0
      * @post size () == $pre (int, size ())
      */  

    public Object peek () {
        return _storage [_top - 1];
    }

    /**
      * @pre size () > 0
      * @post size () == $pre (int, size ()) - 1
      * @concurrency sequential
      */

    public Object pop () {
        return _storage [--_top];
    }

    public int size () {
        return _top;
    }
    
    /** @post $result == (size () == 0) */

    public boolean isEmpty () {
        return _top == 0;
    }
    
    private final static int MAX_SIZE = 100;
    private Object _storage [];
    private int _top;


    //////////

    public static void main (String[] args)
    {
        FixSizeStack stack = new FixSizeStack ();
        stack.push ("one");
        stack.push ("two");
        System.out.println ("size = " + stack.size ());
        System.out.println ("is_empty = " + stack.isEmpty ());
        System.out.println ("peek = " + stack.peek ());
        System.out.println ("pop = " + stack.pop ());
        System.out.println ("pop = " + stack.pop ());
    }
}
