// This class has been written following the specification in the file
// Queue.spec.
// 
// We will use jtest to test this class using BlackBox testing.
// For this example we will ignore the automatic test cases and we will
// create only user defined test cases.
//
// The file Queue.ctp in this directory contains a set of black-box tests
// for the Queue class. Open this file in the "Class Testing UI":
//
// 1) Start jtest and bring up the "Class Testing UI" (Close the "Project
//    Testing UI" if it is open.)
// 2) Open the "Queue.ctp" test specification:
//    2.1) Select MenuBar -> File -> Open.
//    2.2) Browse to the Queue.ctp file located in this directory.
//
// A black-box specification defines a set of inputs and its corresponding
// correct outcomes.
//
// First we are going to see the set of inputs that have been defined:
// (Test Case Generation).
//
// 1) Push the "Class" button in the toolbar.
// 2) Open: "Dynamic Analysis"
//       -> "Test Case Generation"
//       -> "User Defined"
//       -> "Method Inputs"
//
// here we see all the methods of the class and we can open the methods 
// to see the inputs. We see that for each method we run it with both
// an empty queue and a non-empty queue. The "emptyQueue" and 
// "nonEmptyQueue" are methods defined in the local inputs repository.
// You can see their definitions in:
// ... -> "Test Case Generation" -> "Common" -> "Inputs Repository".
//
// The black-box specification should also contain the list of correct
// outcomes for those inputs (Test Case Evaluation). To see them open:
//
// "Test Case Evaluation" -> "Specification and Regression Test Cases".
//
// here we see all the methods of the class and for each method the list
// of test cases that have been generated from the above defined inputs.
// Each test case lists its outcomes. The green arrow means that the
// user has specified those outcomes as the correct ones.
//
//
// Everytime this class is tested with jtest, jtest runs all those
// test cases and checks that the outcomes are correct. If any outcome
// is incorrect it will report an error. 
//
//
// The steps that are necessary to create a black-box specification like
// the one show here are:
//
// 1) Define the inputs in "Test Case Generation" -> "User Defined".
//    1.1) For the example show with this class one would first define two
//         method repository entries "emptyQueue" and "nonEmptyQueue".
//    1.2) Specify the inputs for each method argument using the popup
//         menus associated to the method arguments. Note that the two
//         repository entries "emptyQueue" and "nonEmptyQueue" are
//         available in the popup menus. 
//
// 2) Test the class by pressing the "Start" button in the toolbar.
//    This will create automatically the list of test-cases and corresponding
//    outcomes.
//
// 3) Once 2) is completed. Look at the test-cases run by pushing the
//    "View" button in the toolbar and verify the outcomes:
//    one would look at each TestCase in turn and validate the outcomes
//    using the popup menu associated to each outcome.


package examples.dynamic.blackbox;

public class Queue
{
    public Queue () {
        _first = null;
        _last  = null;
    }
    
    public void add (Object item) {
        QueueItem queue_item = new QueueItem (item);
        if (_first == null) {
            _first = queue_item;
            _last  = queue_item;
        } else {
            _last.setNext (queue_item);
            _last = queue_item;
        }
    } 

    public Object remove () {
        if (isEmpty ())
            throw new EmptyQueueException ();

        QueueItem first = _first;
        first = first.getNext ();
        return first.getItem ();
    }

    public boolean isEmpty () {
        return _first == null;
    }

    public void removeAll () {
        _first = null;
        _last  = null;
    }

    public String toString () {
        StringBuffer buffer = new StringBuffer ("Queue[");
        for (QueueItem next = _first; next != null; next = next.getNext ()) {
            if (next != _first)
                buffer.append (",");
            buffer.append (next.getItem ().toString ());
        }
        buffer.append ("]");
        return buffer.toString ();
    }

    private QueueItem _first;
    private QueueItem _last;

    private class QueueItem {
        QueueItem (Object item) {
            _item = item;
        }
        Object getItem () {
            return _item;
        }
        QueueItem getNext () {
            return _next;
        }
        void setNext (QueueItem next) {
            _next = next;
        }
        private QueueItem _next;
        private Object    _item;
    }
}

class EmptyQueueException extends RuntimeException {}
