// Example: simple BankAccount.
//
// This example shows how jtest automatically builds a test-suite for the
// class and performs automatic regression testing using that test-suite.
//
// The test-suite is build the first time jtest is run on the class:
//
// 1) Run jtest on this class.
//
// 2) View the automatically generated test cases for the "deposit"
//    and "withdraw" methods:
//    2.1) Push the "View" button in the "Class Testing UI" toolbar.^M
//    2.2) Open "Automatic Test Cases" -> "Open deposit and withdraw".^M
//
// Note for example the "Test Case 3" for the "withdraw" method: this
// test case checks that if an amount greater than the current balance
// is attempted to withdraw, the method throws a ProcessingException
// and leaves the account balance untouched. Note that no user
// intervention was needed, jtest automatically generated the inputs
// and selected/stored the outcomes to check.


package examples.dynamic.regression;

public class BankAccount
{
    public BankAccount () {
        _balance = 0;
    }

    public int balance () {
        return _balance;
    }

    public void deposit (int amount) {
        if (amount < 0)
            throw new ProcessingException ("invalid deposit amount: " + amount);
        _balance += amount;
    }

    public void withdraw (int amount) {
        if (amount < 0) {
            throw new ProcessingException (
                  "invalid withdraw amount: " + amount
            );
        }
        if (_balance - amount < 0) {
            throw new ProcessingException (
                  "Maximum withdraw amount is: " + _balance
            );
        }
        _balance -= amount;
    }

    private int _balance;
}

class ProcessingException extends RuntimeException {
    ProcessingException () {super ();}
    ProcessingException (String message) {super (message);}
}
