//
// Example of I/O white-box stubs
//
// This example shows how jtest does white-box testing of code using I/O.
// Using white-box stubs jtest generates return values for the I/O calls
// so that the code is tested and exceptions are generated.
//
// 1) Test this class using the "Class Testing UI" of jtest.
//
// 2) Look at the "Uncaught Runtime Exception" found by jtest:
//    2.1) The error message says that a NullPointerException
//         will happen for the file input shown
//
// 3) Look at the test-cases generated:
//    3.1) Push the "View" button in the toolbar.
//         Note that jtest has generated enough test cases as to get 100%
//         coverage.


package examples.dynamic.stubs.auto;

import java.io.*;
import java.util.*;

public class AccountUpdater
{
    public AccountUpdater (String filename, int num_accounts)
        throws IOException
    {
        FileInputStream fis = new FileInputStream (filename);
        _dis = new DataInputStream (fis);
        _numAccounts = num_accounts;
    }

    public int checkForLowBalances ()
        throws IOException
    {
        int num_low = 0;
        for (int i = 0; i < _numAccounts; i++) {
            Account account = new Account (_dis);
            if (account.getBalance () < 100
            && !account.getName ().equals ("BOSS")) {
                num_low++;
                addToLowBalancesList (account);
            }
        }
        return num_low;
    }

    private void addToLowBalancesList (Account account) {
        _lowBalances.addElement (account); 
    }

    private static class Account {
        Account (DataInputStream dis) throws IOException
        {
            _name    = dis.readUTF ();
            _balance = dis.readInt ();
        } 
        void write (DataOutputStream dos) throws IOException
	{
            dos.writeUTF (_name);
            dos.writeInt (_balance);
        }
        String getName () {return _name;}
        int    getBalance () {return _balance;}
        private String _name;
        private int    _balance;
    }

    private DataInputStream _dis;
    private int _numAccounts;
    private Vector _lowBalances;
}
