//
// Example of java.net white-box stubs
//
// This example shows how jtest does white-box testing of code using Sockets.
// Using white-box stubs jtest generates exceptions and return values
// so that the code is tested and exceptions are generated.
//
// 1) Load the test "EchoSocket.ctp".
//
// 2) Run the test by pushing the "Start" button in the "Class Testing UI".
//
// 3) Look at the "Uncaught Runtime Exception" found by jtest:
//    3.1) The error message says that a NullPointerException
//         will happen for the input shown.
//
// 4) Look at the test-cases generated:
//    4.1) Push the "View" button in the toolbar.
//         Note that jtest has generated enough test cases as to get 100%
//         coverage. Note also that for "Test Case 3" jtest has generated
//         the "EXIT" value for "readLine ()".


package examples.dynamic.stubs.auto;

import java.io.*;
import java.net.*;

public class EchoSocket extends Thread
{
    EchoSocket (ServerSocket server_socket)
        throws IOException
    {
        _socket = server_socket.accept ();
        InputStream  input  = _socket.getInputStream ();
        OutputStream output = _socket.getOutputStream ();
        _in  = new BufferedReader (new InputStreamReader (input));
        _out = new PrintWriter (output);
    }

    public void run ()
    {
        _out.println ("Echo: enter EXIT to exit");

        try {
            while (true) {
                String line = _in.readLine ();
                _out.println ("Echo: " + line);
                if (line.equals ("EXIT"))
                    break;
            }
        } catch (IOException e) {
            e.printStackTrace ();
        }

        try {
            _socket.close ();
        } catch (IOException e) {}
    }

    private Socket         _socket;
    private BufferedReader _in;
    private PrintWriter    _out;
}
