//
// Example of SQL white-box stubs
//
// This example shows how jtest does white-box testing of code using SQL.
// Using white-box stubs jtest generates return values for the SQL calls
// so that the code is tested and exceptions are generated.
//
// 1) Test this class using the "Class Testing UI".
//
// 2) Look at the "Uncaught Runtime Exception" found by jtest:
//    2.1) The error message says that a NullPointerException
//         will happen for the string that returned from SQL method. 
//
// 3) Look at the test-cases generated:
//    3.1) Push the "View" button in the toolbar.
//         Note that jtest has generated enough test cases as to get 100%
//         coverage.
//


package examples.dynamic.stubs.auto;

import java.sql.*;
import java.util.*;

public class ResultSetExample 
{
    public ResultSetExample (String url, String user, String password)
        throws SQLException
    {
        _connection = DriverManager.getConnection (url, user, password);
        _db = _connection.getMetaData ();
    }

    public void printColumnNameAndSize (String tableName) 
       throws SQLException 
    {
        if (tableName == null)
            return;

        ResultSet columns = getColumnSet ("", "DBA", tableName, "");
        String s = "";

        while (columns.next ()) {
            String colName = columns.getString (COLUMN_NAME);
            s += colName + " (" +colName.length () +")\n";
        }

        System.out.println ("Column names for " +tableName + " : \n" + s); 
    }
    
    private ResultSet getColumnSet (
          String catalog
        , String schemaPattern
        , String tableName
        , String columnNamePattern
    )
        throws SQLException
    {
        return _db.getColumns (
              catalog
            , schemaPattern
            , tableName
            , columnNamePattern
        );
    }

    private static final int COLUMN_NAME = 4;
    private DatabaseMetaData _db;
    private Connection       _connection;
}
