//
// Example for "Object jtest.Stubs.makeStubObject (Class cl)"
//
// The 'makeStubObject' creates an stub object for the class 'cl'.
// Stub objects are very useful when writting user defined stubs. A stub
// object is like any other object, with the following differences:
//
// a) The stub object can be an instance of an interface. I.e. the following
//    creates an instance of "Enumeration":
//
//       Enumeration enum = makeStubObject (Enumeration.class);
//
// b) Any method invocation or field reference is an stub even if no stub has
//    been defined for it. If no stub has been defined for it a default stub
//    returning the default initialization value for the method return type
//    or field type is used (i.e. "null" for Object, "0.0d" for double, ...).
//
// Stub objects are necessary to be able to test classes that use interfaces
// for which an implementation has not been written yet. They can be used
// whenever an object of the interface class needs to be created.
// They can also be used whenever one wants to create an object of a given type
// without having to call any specific constructor (i.e. instead of using
// "new java.io.FileInputStream ("what to put here?)", one can use:
// "(FileInputStream) JT.makeStubObject (java.io.FileInputStream.class)", this
// creates a FileInputStream object, but no constructor is called to initialize
// it.
//
//
// For this example the 'makeStubObject()' method has been used in two places:
//
// a) In the "stubs()" method below.
//
// b) As the user defined input for argument "t1" of method "count".
//   
//
// To see how this is done:
//
// 1) Load "StubObject.ctp" in the "Class Testing UI".
//
// 2) Look at the user defined input for argument "t1".
//    2.1) Push "Class" in the toolbar.
//    2.2) Open "Class Test Parameters" -> "Dynamic Analysis" -> "Test Case
//         Generation" -> "User Defined" -> "Method Inputs" -> "count" -> "ARG1"
//         Note that in here makeStubObject needs to be invoked as:
//         "JT.makeStubObject (...)".   
//
// 3) Push "Start" in the toolbar to execute the user defined test cases.
//
// 4) Once finished look at the "Test Case Input" for the test cases run.
//    4.1) Push "View" in the toolbar.
//    4.2) Open "View Test Cases" -> "User Defined Test Cases" -> "Method
//         Inputs" -> "count" -> "Test Case 1".
//    4.3) Note that even though we have not defined stubs for the "numSlices"
//         method, Jtest automatically generates an stub because the Toaster
//         object was an object created with "makeStubObject()".
//  


package examples.dynamic.stubs.userdef;

import java.lang.reflect.*;

public class StubObjects
{
    public static int count (Toaster t1)
    {
        Toaster t2 = t1.copy ();
        int num = 0;
        if (t1.done ())
            num += t1.numSlices ();
        if (t2.done ())
            num += t2.numSlices ();
        return num;
    } 
}


class StubObjectsStubs extends jtest.Stubs
{
    public static Object stubs (Method method)
    {
        String method_name = method.getName ();

        if (method_name.equals ("copy"))
            return makeStubObject (Toaster.class);

        if (method_name.equals ("done"))
            return Boolean.TRUE;

        return NO_STUB_GENERATED;
    }
}
