// This example shows how to make jtest not to report errors for the
// conditions that violate the assert statements.


package examples.dynamic.whitebox;

public class Assert
{
    public void method1 (int size) {
        assert1 (size >= 0);
        int [] array = new int [size];
        //...
    } 

    public void method2 (int size) {
        assert2 (size >= 0, "negative size");
        int [] array = new int [size];
        //...
    } 

    // To prevent jtest from reporting the errors when using this form of
    // assert one just needs to add a suppression for "AssertFailedException"
    // in the jtest Suppression Table. 

    private static void assert1 (boolean condition) {
        if (condition == false)
            throw new AssertFailedException ();
    }


    // If using this form of assert, jtest will never report an error

    private static void assert2 (boolean condition, String message) {
        if (condition == false)
            throw new AssertFailedException (message);	
    }
}

class AssertFailedException extends RuntimeException {
    AssertFailedException () {super ();}
    AssertFailedException (String message) {super (message);}
}
