// Supress example: Direct IllegalArgumentExceptions


package examples.dynamic.whitebox;

public class Suppress1
{
    // If the "Direct IllegalArgumentExceptions" suppression cathegory is on
    // jtest will not report the IllegalArgumentException that can
    // occur when calling this constructor with a positive 'arg1'.
    //
    // Throwing an IllegalArgumentException is the correct behavior for
    // a method or contructor that is passed an illegal argument.
    //
    // Note that the "Explicitly Thrown Exceptions" cathegory also needs
    // to be off for this exception to be reported.

    public Suppress1 (int arg1, int arg2)
    {
        if (arg1 > 0)
            throw new IllegalArgumentException ("arg1 greater that zero");

        // ...
    }

    // Note that jtest will not report an exception here either if the
    // suppression cathegory is on. When testing a method, jtest will
    // not report the IllegalArgumentException if the exception is
    // thrown by another method called with the same name and in the
    // same class. 
    // The reason is that in this way only the most generic method or
    // contructor needs to do the error checking.

    public Suppress1 (int arg1) {
        this (arg1, 10);
    }

    // Also jtest will not report the exception if an argument is passed
    // to another method and that method throws an IllegalArgumentException
    // on that argument:

    public static void method (int arg) {
        checkArgs (arg);
    }

    private static void checkArgs (int arg) {
        if (arg == 7)
            throw new IllegalArgumentException ("arg is 7");
    }
}
