// Supress example: Direct NullPointerException


package examples.dynamic.whitebox;

public class Suppress3
{
    public static int smethod (String string) {
        // If the "Direct NullPointerException" suppression cathegory is on
        // jtest will not report the NullPointerException that can
        // occur when calling this method with a 'null' 'string'.
        //
        // The most common fix for this case would be to check the argument
        // and throw an IllegalArgumentException if it is 'null'.
        // This kind of exceptions are quite common and the fix doesn't
        // give much more information to the user of the method that just
        // letting the code throw a NullPointerException.

        return string.length ();
    }

    public int method () {
        // This method will throw a NullPointerException if the constructor
        // for the 'this' object was created with a 'null' 'text'.
        // This exception will be reported even if the
        // "Direct NullPointerException" suppression cathegory is on.
        //
        // In the general case the reason of the exceptions may be more
        // difficult to track. If the constructor is not supposed to accept
        // a 'null' 'text' then it should throw an IllegalArgumentException
        // instead of creating an invalid object, not complaining about it
        // letting the program keep running with that invalid object.

        return _text.length (); 
    }

    Suppress3 (String text) {
        _text = text;
    }

    String _text;
}
