// Example for Design by Contract support in Jtest.
//
// Simple Stack implementation using a fix size array.
//
// @pre, @post and @invariant contracts are used to specify the assumed
// preconditions and the specification for this class.
//
// A couple of bugs have been introduced on purpose in this class (see the
// lines labeled "//CORRECT: ".
//
// 1) Test this class with Jtest using Dynamic Analysis.
//
// 2) See the @post violations generated due to the bugs introduced.


package examples.eval.project;

/** @invariant size () >= 0 */

public class FixSizeStack
{
    /** @post size () == 0 */

    public FixSizeStack ()
    {
        _storage = new Object [MAX_SIZE];
        _top = 0;
    }

    /**
      * @pre size () < MAX_SIZE - 1
      * @post peek () == object
      * @post size () == $pre (int, size ()) + 1
      */

    public void push (Object object) {
        _storage [_top++] = object;
    }

    /**
      * @pre size () > 0
      * @post size () == $pre (int, size ())
      */  

    public Object peek () {
        return _storage [_top - 1];
    }

    /**
      * @pre size () > 0
      * @post size () == $pre (int, size ()) - 1
      */

    public Object pop ()
    {
        //CORRECT: return _storage [--_top];
                   return _storage [_top--];
    }

    public int size () {
        return _top;
    }
    
    /** @post $result == (size () == 0) */

    public boolean isEmpty ()
    {
        //CORRECT: return _top == 0;
                   return _top != 0;
    }
    
    private final static int MAX_SIZE = 100;
    private Object _storage [];
    private int _top;
}
